--[[
    Aaron MacDonald
    August 28, 2010
]]

local GRAPHICS_DIR = 'graphics/'
local GRAPHICS_TYPE = '.png'

local FONT_SIZE = 24
local FONT_FILE = 'fonts/Commodore Rounded v1-1.ttf'

----------------------------------------------------------------------

gfx = {}

function gfx.loadImage(spriteName, haveExtension)
    local path = GRAPHICS_DIR .. spriteName
    if not haveExtension then
        path = path .. GRAPHICS_TYPE
    end
    local image = love.graphics.newImage(path)

    return image
end

function gfx.createSprite(spriteName, haveExtension)
    local image = gfx.loadImage(spriteName, haveExtension)

    return { drawable = image,
             width = image:getWidth(),
             height = image:getHeight(),
             offsetX = image:getWidth() / 2,
             offsetY = image:getHeight() / 2 }
end

function gfx.createParticleSprite(emmiter, particleSprite,
  particleLifeMax, particleSpeedMax, particleSizeMax)
    local w = (particleLifeMax * particleSpeedMax)
      + (particleSprite:getWidth() * particleSizeMax)
    local h = (particleLifeMax * particleSpeedMax)
      + (particleSprite:getHeight() * particleSizeMax)

    return { drawable = emmiter,
             width = w,
             height = h,
             offsetX = nil,
             offsetY = nil }
end

function gfx.loadFont()
    love.graphics.setFont(FONT_FILE, FONT_SIZE)
end

function gfx.fontHeight(textAbove)
    local result = love.graphics.getFont():getHeight()

    if textAbove then
        result = result + love.graphics.getFont():getLineHeight()
    end

    return result
end

function gfx.drawSprite(sprite, x, y, angle, scaleX, scaleY)
    love.graphics.draw(sprite.drawable, x, y, angle or 0,
      scaleX or 1, scaleY or 1, sprite.offsetX, sprite.offsetY)
end

function gfx.drawWrapped(sprite, x, y,
  spriteWidth, spriteHeight, regionWidth, regionHeight,
  offsetX, offsetY, angle, scaleX, scaleY)

    local ox = offsetX or 1
    if scaleX then
        ox = ox * scaleX
    end

    local oy = offsetY or 1
    if scaleY then
        oy = oy * scaleY
    end

    love.graphics.draw(sprite, x, y, angle or 0,
      scaleX or 1, scaleY or 1, ox, oy)

    -- Wraparound

    local width = spriteWidth

    if scaleX then
        width = width * scaleX
    end
    if not offsetX then
        width = width / 2
    end

    local height = spriteHeight

    if scaleY then
        height = height * scaleY
    end
    if not offsetY then
        height = height / 2
    end

    local secondX, secondY

    if x < width then
        secondX = regionWidth + x
    elseif (regionWidth - x) < width then
        secondX = x - regionWidth
    end

    if y < height then
        secondY = regionHeight + y
    elseif (regionHeight - y) < height then
        secondY = y - regionHeight
    end

    if secondX or secondY then
        secondX = secondX or x
        secondY = secondY or y

        assert(secondX and secondY
          and (secondX ~= x) or (secondY ~= y))

        love.graphics.draw(sprite, secondX, secondY, angle or 0,
          scaleX or 1, scaleY or 1, ox, oy)
    end
end

function gfx.drawWrappedSprite(sprite, x, y,
  regionWidth, regionHeight, angle, scaleX, scaleY)
    gfx.drawWrapped(sprite.drawable, x, y,
      sprite.width, sprite.height, regionWidth, regionHeight,
      sprite.offsetX, sprite.offsetY, angle, scaleX, scaleY)
end

function gfx.interpolatedColour(colourA, colourB, fade)
    local invFade = 1 - fade

    local red = (colourA[1] * fade) + (colourB[1] * invFade)
    local green = (colourA[2] * fade) + (colourB[2] * invFade)
    local blue = (colourA[3] * fade) + (colourB[3] * invFade)

    return {red, green, blue}
end

function gfx.setColourWhite()
    love.graphics.setColor(255, 255, 255)
end